//#define DEBUG

#ifndef MIN
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#endif

#ifndef MAX
#define MAX(a,b) ((a) > (b) ? (a) : (b))
#endif

#define AddIntConstant(c) if (PyModule_AddIntConstant(m, #c, c) < 0) return;

#ifdef DEBUG

#define TraceMessage(_msg)                      \
{                                               \
    printf("%s\n", _msg);                       \
}

#define TraceMethodEnter(_name, _obj)                           \
{                                                               \
    PyObject *repr = NULL;                                      \
    char *repr_str = NULL;                                      \
                                                                \
    if (_obj) {                                                 \
        repr = _obj->ob_type->tp_repr((PyObject *)_obj);        \
        repr_str = PyString_AsString(repr);                     \
    }                                                           \
    printf("%s: %s\n", _name, repr_str);                        \
    Py_XDECREF(repr);                                           \
}

#define TraceMethodLeave(_name, _obj)                           \
{                                                               \
    PyObject *repr = NULL;                                      \
    char *repr_str = NULL;                                      \
                                                                \
    if (_obj) {                                                 \
        repr = _obj->ob_type->tp_repr((PyObject *)_obj);        \
        repr_str = PyString_AsString(repr);                     \
    }                                                           \
    printf("%s: %s\n", _name, repr_str);                        \
    Py_XDECREF(repr);                                           \
}

#define TraceObjNewEnter(_name, _tp)            \
{                                               \
    PyTypeObject *tp = _tp;                     \
    if (tp != NULL)                             \
        printf("%s %s\n", _name, tp->tp_name);  \
    else                                        \
        printf("%s\n", _name);                  \
}


#define TraceObjNewLeave(_name, _obj)                                   \
{                                                                       \
    PyObject *repr = NULL;                                              \
                                                                        \
    if ((repr = _obj->ob_type->tp_repr((PyObject *)_obj))) {            \
        printf("%s: returns %s\n", _name, PyString_AsString(repr));     \
        Py_DECREF(repr);                                                \
    }                                                                   \
}

#else
#define TraceMessage(_msg)
#define TraceMethodEnter(_name, _obj)
#define TraceMethodLeave(_name, _obj)
#define TraceObjNewEnter(_name, _tp)
#define TraceObjNewLeave(_name, _obj)
#endif
